==> femmt-0.2.0-py3-none-any.whl: femmt/thermal/thermal_simulation.py <==
import gmsh
from os import path
from onelab import onelab
from .thermal_functions import *
from .thermal_classes import ConstraintPro, FunctionPro, GroupPro, ParametersPro

def create_case(boundary_regions, boundary_physical_groups, boundary_temperatures, boundary_flags, k_case, function_pro: FunctionPro, parameters_pro: ParametersPro, group_pro: GroupPro, constraint_pro: ConstraintPro):
    """
    Sets boundary conditions and material parameters for the case around the core.

    TODO Set docstring
    
    """
    group_pro.add_regions(boundary_regions)
    parameters_pro.add_to_parameters(boundary_temperatures)
    parameters_pro.add_to_parameters(boundary_flags)
    constraint_pro.add_boundary_constraint([x for x in zip(boundary_flags.keys(), boundary_regions.keys(), boundary_temperatures.keys())])
    
    k = {"case": k_case}
    q_vol = {"case": 0}

    function_pro.add_dicts(k, q_vol)
    group_pro.add_regions({"case": f"{{{str(boundary_physical_groups)[1:-1]}}}"})

    dim_tags = []
    for physical_group in boundary_physical_groups:
        for tag in gmsh.model.getEntitiesForPhysicalGroup(2, physical_group):
            dim_tags.append([2, tag])

    return dim_tags

def create_background(background_tag, k_air, function_pro: FunctionPro, group_pro: GroupPro):
    k_air = {"air": k_air}
    q_vol_air = {"air": 0}

    function_pro.add_dicts(k_air, q_vol_air)
    group_pro.add_regions({"air": background_tag})
    
    return [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, background_tag)]

def create_core_and_air_gaps(core_tag, k_core, core_area, core_losses, air_gaps_tag, k_air_gaps, function_pro: FunctionPro, group_pro: GroupPro):
    heat_flux = core_losses/core_area
    
    if air_gaps_tag is not None:
        k = {
        "core": k_core,
        "air_gaps": k_air_gaps 
        }
        q_vol = {
            "core": heat_flux,
            "air_gaps": 0
        }
        group_pro.add_regions({
                "core": core_tag,
                "air_gaps": air_gaps_tag
        })
        function_pro.add_dicts(k, q_vol)
    else:
        k = {"core": k_core}
        q_vol = {"core": heat_flux}
        group_pro.add_regions({"core": core_tag})
        function_pro.add_dicts(k, q_vol)
        
    

    return [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, core_tag)], [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, air_gaps_tag)] if air_gaps_tag is not None else None

def create_windings(winding_tags, k_windings, winding_losses, conductor_radii, function_pro: FunctionPro, group_pro: GroupPro):
    q_vol = {}
    k = {}
    regions = {}
    windings_total_str = "{"
    entities = []

    for winding_index, winding in enumerate(winding_tags):
        if winding is not None and len(winding) > 0:
            for index, tag in enumerate(winding):
                name = f"winding_{winding_index}_{index}"
                windings_total_str += f"{name}, "
                q_vol[name] = calculate_heat_flux_round_wire(winding_losses[winding_index][index], conductor_radii[winding_index])
                k[name] = k_windings
                regions[name] = tag
                for entity in gmsh.model.getEntitiesForPhysicalGroup(2, tag):
                    entities.append(entity)
                
    # Needs to be added. [:-2] removes the last ', '
    regions["windings_total"] = windings_total_str[:-2] + "}"

    function_pro.add_dicts(k, q_vol)
    group_pro.add_regions(regions)

    return [[2, tag] for tag in entities]
    
def simulate(onelab_folder_path, mesh_file, solver_file):
    c = onelab.client(__file__)

    # Run simulations as sub clients (non blocking??)
    mygetdp = path.join(onelab_folder_path, "getdp")
    c.runSubClient("myGetDP", mygetdp + " " + solver_file + " -msh " + mesh_file + " -solve analysis -v2")

def run_thermal(onelab_folder_path, model_mesh_file_path, results_log_file_path, 
    tags_dict, thermal_conductivity_dict, boundary_temperatures, 
    boundary_flags, boundary_physical_groups, core_area, conductor_radii, 
    show_results, pretty_colors = False, show_before_simulation = False):
    """
    Runs a thermal simulation.
    
    :param onelab_folder_path: Path to the onelab directory
    :param model_mesh_file_path: Path to the .msh file generated by the magnetic simulation
    :oaran results_log_file_path: Path to the results log file generated by the magnetic simulation
    :param tags_dict: Dictionary containing the tags of the case, air, core and windings
    :param thermal_conductivity_dict: Dictionary containing the thermal conductivity material parameters for case, air, core and windings
    :param mesh_size: Settings the mesh size for the case wihich will be constructed around the core
    :param core_area: Area of the cross-section of the core
    :param conductor_radii: List of the radius for each winding 
    :param show_results: Boolean - Set true when the results shall be shown in a gmsh window
    :param pretty_colors: Boolean - Set true if a specified colorization should be applied
    :param show_before_simulation: -  Set true if the mesh should be shown before running the thermal simulation (e.g. to see the colorization)

    :param return: -
    """
    
    losses = read_results_log(results_log_file_path)

    # Realtive paths
    solver_folder = path.join(path.dirname(__file__), "solver")
    map_pos_file = path.join(solver_folder, "map.pos")
    thermal_template_file = path.join(solver_folder, "Thermal.pro")
    parameters_file = path.join(solver_folder, "Parameters.pro")
    function_file = path.join(solver_folder, "Function.pro")
    group_file = path.join(solver_folder, "Group.pro")
    constraint_file = path.join(solver_folder, "Constraint.pro")
    
    gmsh.initialize()
    gmsh.open(model_mesh_file_path)
    
    # Create file wrappers
    parameters_pro = ParametersPro()
    function_pro = FunctionPro()
    group_pro = GroupPro()
    constraint_pro = ConstraintPro()

    # Extract losses
    winding_losses = []
    for i in range(1, 3):
        key = f"Winding_{i}"
        inner_winding_list = []
        if key in losses:
            for loss in losses[key]["Turns"]:
                inner_winding_list.append(loss)
        winding_losses.append(inner_winding_list)

    core_losses = losses["Core_Eddy_Current"]

    case_dim_tags = create_case(tags_dict["boundary_regions"], boundary_physical_groups, boundary_temperatures, boundary_flags, thermal_conductivity_dict["case"], function_pro, parameters_pro, group_pro, constraint_pro)
    background_dim_tags = create_background(tags_dict["background_tag"], thermal_conductivity_dict["air"], function_pro, group_pro)
    core_dim_tags, air_gaps_dim_tags = create_core_and_air_gaps(tags_dict["core_tag"], thermal_conductivity_dict["core"], core_area, core_losses, tags_dict["air_gaps_tag"], thermal_conductivity_dict["air_gaps"], function_pro, group_pro)
    windings_dim_tags = create_windings(tags_dict["winding_tags"], thermal_conductivity_dict["winding"], winding_losses, conductor_radii, function_pro, group_pro)

    gmsh.model.geo.synchronize()

    # Colorize
    if pretty_colors:
        gmsh.model.setColor(case_dim_tags, 50, 50, 50)
        gmsh.model.setColor(background_dim_tags, 255, 255, 255)
        gmsh.model.setColor(core_dim_tags, 110, 110, 110)
        gmsh.model.setColor(windings_dim_tags, 192, 28, 40)
        if air_gaps_dim_tags is not None:
            gmsh.model.setColor(air_gaps_dim_tags, 203, 156, 190)
        
    gmsh.model.mesh.generate()
    gmsh.write(model_mesh_file_path)

    if show_before_simulation:
        gmsh.fltk.run()

    gmsh.finalize()

    # Create files
    parameters_pro.create_file(parameters_file)
    function_pro.create_file(function_file)
    group_pro.create_file(group_file, tags_dict["air_gaps_tag"] != None)
    constraint_pro.create_file(constraint_file)

    simulate(onelab_folder_path, model_mesh_file_path, thermal_template_file)

    if show_results:
        gmsh.initialize()
        gmsh.open(map_pos_file)
        gmsh.fltk.run()
        gmsh.finalize()

if __name__ == "__main__":

    # Simulation parameters
    pretty_colors = True
    show_before_simulation = False

    # Currently absolute paths
    onelab_folder_path = r"C:\Uni\Bachelorarbeit\onelab-Windows64"
    results_log_file_path = r"C:\Uni\Bachelorarbeit\github\FEM_Magnetics_Toolbox\femmt\results\result_log.json"
    model_mesh_file_path = r"C:\Uni\Bachelorarbeit\github\FEM_Magnetics_Toolbox\femmt\thermal\thermal_mesh.msh"
    winding_tags = [list(range(4000, 4036)), list(range(7000, 7011)), None]
    
    tags = {
        "core_tag": 2000,
        "background_tag": 1000,
        "winding_tags": winding_tags,
        "core_line_tags": [4, 3, 2],
        "core_point_tags": [5, 4, 3, 2], # Order: top left, top right, bottom right, bottom left
        "air_gaps_tag": 1001,
        "boundary_regions": {
            "BOUNDARY_TOP"            : 1112,
            "BOUNDARY_TOP_RIGHT"      : 1113,
            "BOUNDARY_RIGHT_TOP"      : 1114,
            "BOUNDARY_RIGHT"          : 1115,
            "BOUNDARY_RIGHT_BOTTOM"   : 1116,
            "BOUNDARY_BOTTOM_RIGHT"   : 1117,
            "BOUNDARY_BOTTOM"         : 1118
        }
    }
    thermal_conductivity_dict = {
        "air": 0.0263,
        "case": 0.3,
        "core": 5,
        "winding": 400,
        "air_gaps": 0.0263 # Air
    }

    boundary_temperatures = {
        "value_boundary_top": 283,
        "value_boundary_top_right": 283,
        "value_boundary_right_top": 283,
        "value_boundary_right": 283,
        "value_boundary_right_bottom": 283,
        "value_boundary_bottom_right": 283,
        "value_boundary_bottom": 283
    }

    boundary_flags = {
        "flag_boundary_top": 1,
        "flag_boundary_top_right": 1,
        "flag_boundary_right_top": 1,
        "flag_boundary_right": 1,
        "flag_boundary_right_bottom": 1,
        "flag_boundary_bottom_right": 1,
        "flag_boundary_bottom": 1
    }

    boundary_physical_groups = [] # Only needed for colorization

    core_area = 0.00077
    conductor_radii = [0.0011, 0.0011]
    show_results = True

    thermal_parameters = {
        "onelab_folder_path": onelab_folder_path,
        "model_mesh_file_path": model_mesh_file_path,
        "results_log_file_path": results_log_file_path,
        "tags_dict": tags,
        "thermal_conductivity_dict": thermal_conductivity_dict,
        "boundary_temperatures": boundary_temperatures,
        "boundary_flags": boundary_flags,
        "boundary_physical_groups": boundary_physical_groups,
        "core_area": core_area,
        "conductor_radii": conductor_radii,
        "show_results": show_results,
        "pretty_colors": False,
        "show_before_simulation": False
    }

    run_thermal(**thermal_parameters)

==> femmt-0.2.1-py3-none-any.whl: femmt/thermal/thermal_simulation.py <==
import gmsh
from os import path
from onelab import onelab
from .thermal_functions import *
from .thermal_classes import ConstraintPro, FunctionPro, GroupPro, ParametersPro

def create_case(boundary_regions, boundary_physical_groups, boundary_temperatures, boundary_flags, k_case, function_pro: FunctionPro, parameters_pro: ParametersPro, group_pro: GroupPro, constraint_pro: ConstraintPro):
    """
    Sets boundary conditions and material parameters for the case around the core.

    TODO Set docstring
    
    """
    group_pro.add_regions(boundary_regions)
    parameters_pro.add_to_parameters(boundary_temperatures)
    parameters_pro.add_to_parameters(boundary_flags)
    constraint_pro.add_boundary_constraint([x for x in zip(boundary_flags.keys(), boundary_regions.keys(), boundary_temperatures.keys())])
    
    k = {
        "case_top": k_case["top"],
        "case_top_right": k_case["top_right"],
        "case_right": k_case["right"],
        "case_bot_right": k_case["bot_right"],
        "case_bot": k_case["bot"] 
    }
    q_vol = {
        "case_top": 0,
        "case_top_right": 0,
        "case_right": 0,
        "case_bot_right": 0,
        "case_bot": 0 
    }

    function_pro.add_dicts(k, q_vol)
    group_pro.add_regions({
        "case_top": boundary_physical_groups["top"],
        "case_top_right": boundary_physical_groups["top_right"],
        "case_right": boundary_physical_groups["right"],
        "case_bot_right": boundary_physical_groups["bot_right"],
        "case_bot": boundary_physical_groups["bot"] 
    })

    dim_tags = []
    for key in boundary_physical_groups:
        for tag in gmsh.model.getEntitiesForPhysicalGroup(2, boundary_physical_groups[key]):
            dim_tags.append([2, tag])

    return dim_tags

def create_isolation(isolation_tag, k_iso, function_pro: FunctionPro, group_pro: GroupPro):
    k_iso = {"isolation": k_iso}
    q_vol_iso = {"isolation": 0}

    function_pro.add_dicts(k_iso, q_vol_iso)
    group_pro.add_regions({"isolation": isolation_tag})

    return None
    return [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, isolation_tag)]

def create_background(background_tag, k_air, function_pro: FunctionPro, group_pro: GroupPro):
    k_air = {"air": k_air}
    q_vol_air = {"air": 0}

    function_pro.add_dicts(k_air, q_vol_air)
    group_pro.add_regions({"air": background_tag})
    
    return [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, background_tag)]

def create_core_and_air_gaps(core_tag, k_core, core_area, core_losses, air_gaps_tag, k_air_gaps, function_pro: FunctionPro, group_pro: GroupPro):
    heat_flux = core_losses/core_area
    print(heat_flux)
    if air_gaps_tag is not None:
        k = {
            "core": k_core,
            "air_gaps": k_air_gaps 
        }
        q_vol = {
            "core": heat_flux,
            "air_gaps": 0
        }
        group_pro.add_regions({
                "core": core_tag,
                "air_gaps": air_gaps_tag
        })
        function_pro.add_dicts(k, q_vol)
    else:
        k = {"core": k_core}
        q_vol = {"core": heat_flux}
        group_pro.add_regions({"core": core_tag})
        function_pro.add_dicts(k, q_vol)
        
    

    return [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, core_tag)], [[2, tag] for tag in gmsh.model.getEntitiesForPhysicalGroup(2, air_gaps_tag)] if air_gaps_tag is not None else None

def create_windings(winding_tags, k_windings, winding_losses, conductor_radii, wire_distances, function_pro: FunctionPro, group_pro: GroupPro):
    q_vol = {}
    k = {}
    regions = {}
    windings_total_str = "{"
    entities = [[], []]

    for winding_index, winding in enumerate(winding_tags):
        if winding is not None and len(winding) > 0:
            for index, tag in enumerate(winding):
                name = f"winding_{winding_index}_{index}"
                windings_total_str += f"{name}, "
                q_vol[name] = calculate_heat_flux_round_wire(winding_losses[winding_index][index], conductor_radii[winding_index], wire_distances[winding_index][index])
                print(q_vol[name])
                k[name] = k_windings
                regions[name] = tag
                for entity in gmsh.model.getEntitiesForPhysicalGroup(2, tag):
                    entities[winding_index].append(entity)
                
    # Needs to be added. [:-2] removes the last ', '
    regions["windings_total"] = windings_total_str[:-2] + "}"

    function_pro.add_dicts(k, q_vol)
    group_pro.add_regions(regions)

    return [[[2, tag] for tag in entities[0]], [[2, tag] for tag in entities[1]]]
    
def simulate(onelab_folder_path, mesh_file, solver_file):
    c = onelab.client(__file__)

    # Run simulations as sub clients (non blocking??)
    mygetdp = path.join(onelab_folder_path, "getdp")
    c.runSubClient("myGetDP", mygetdp + " " + solver_file + " -msh " + mesh_file + " -solve analysis -v2")

def run_thermal(onelab_folder_path, results_folder_path, model_mesh_file_path, results_log_file_path, 
    tags_dict, thermal_conductivity_dict, boundary_temperatures, 
    boundary_flags, boundary_physical_groups, core_area, conductor_radii, wire_distances,
    show_results, pretty_colors = False, show_before_simulation = False):
    """
    Runs a thermal simulation.
    
    :param onelab_folder_path: Path to the onelab directory
    :param model_mesh_file_path: Path to the .msh file generated by the magnetic simulation
    :oaran results_log_file_path: Path to the results log file generated by the magnetic simulation
    :param tags_dict: Dictionary containing the tags of the case, air, core and windings
    :param thermal_conductivity_dict: Dictionary containing the thermal conductivity material parameters for case, air, core and windings
    :param mesh_size: Settings the mesh size for the case wihich will be constructed around the core
    :param core_area: Area of the cross-section of the core
    :param conductor_radii: List of the radius for each winding 
    :param wire_distances: List of the outer radius for each winding
    :param show_results: Boolean - Set true when the results shall be shown in a gmsh window
    :param pretty_colors: Boolean - Set true if a specified colorization should be applied
    :param show_before_simulation: -  Set true if the mesh should be shown before running the thermal simulation (e.g. to see the colorization)

    :param return: -
    """

    # Initial Clearing of gmsh data
    gmsh.clear()
    
    losses = read_results_log(results_log_file_path)

    # Relative paths
    map_pos_file = path.join(results_folder_path, "thermal.pos")
    influx_pos_file = path.join(results_folder_path, "thermal_influx.pos")
    material_pos_file = path.join(results_folder_path, "thermal_material.pos")
    solver_folder_path = path.join(os.path.dirname(__file__), "solver")
    thermal_template_file = path.join(solver_folder_path, "Thermal.pro")
    parameters_file = path.join(solver_folder_path, "Parameters.pro")
    function_file = path.join(solver_folder_path, "Function.pro")
    group_file = path.join(solver_folder_path, "Group.pro")
    constraint_file = path.join(solver_folder_path, "Constraint.pro")

    if not gmsh.isInitialized():
        gmsh.initialize()
    gmsh.open(model_mesh_file_path)
    
    # Create file wrappers
    parameters_pro = ParametersPro()
    function_pro = FunctionPro()
    group_pro = GroupPro()
    constraint_pro = ConstraintPro()

    parameters_pro.add_to_parameters({
        "thermal_file": map_pos_file.replace("\\", "/"),
        "thermal_influx_file": influx_pos_file.replace("\\", "/"),
        "thermal_material_file": material_pos_file.replace("\\", "/")
    })

    # Extract losses
    winding_losses = []
    for i in range(1, 3):
        key = f"winding{i}"
        inner_winding_list = []
        if key in losses:
            for winding in losses[key]["turns"]:
                inner_winding_list.append(winding)
        winding_losses.append(inner_winding_list)

    core_losses = losses["core"]

    # TODO All those pro classes could be used as global variables
    case_dim_tags = create_case(tags_dict["boundary_regions"], boundary_physical_groups, boundary_temperatures, boundary_flags, thermal_conductivity_dict["case"], function_pro, parameters_pro, group_pro, constraint_pro)
    background_dim_tags = create_background(tags_dict["background_tag"], thermal_conductivity_dict["air"], function_pro, group_pro)
    core_dim_tags, air_gaps_dim_tags = create_core_and_air_gaps(tags_dict["core_tag"], thermal_conductivity_dict["core"], core_area, core_losses, tags_dict["air_gaps_tag"], thermal_conductivity_dict["air_gaps"], function_pro, group_pro)
    windings_dim_tags = create_windings(tags_dict["winding_tags"], thermal_conductivity_dict["winding"], winding_losses, conductor_radii, wire_distances, function_pro, group_pro)
    isolation_dim_tags = create_isolation(tags_dict["isolations_tag"], thermal_conductivity_dict["isolation"], function_pro, group_pro)

    gmsh.model.geo.synchronize()

    # Colorize
    if pretty_colors:
        gmsh.model.setColor(case_dim_tags, 50, 50, 50)
        gmsh.model.setColor(background_dim_tags, 193, 193, 193)
        gmsh.model.setColor(core_dim_tags, 58, 58, 58)
        gmsh.model.setColor(windings_dim_tags[0], 6, 213, 6)
        gmsh.model.setColor(windings_dim_tags[1], 213, 6, 6)
        if air_gaps_dim_tags is not None:
            gmsh.model.setColor(air_gaps_dim_tags, 255, 171, 6)
        if isolation_dim_tags is not None:
            gmsh.model.setColor(isolation_dim_tags, 109, 109, 109)
        
    gmsh.model.mesh.generate()
    gmsh.write(model_mesh_file_path)

    if show_before_simulation:
        gmsh.fltk.run()

    # Create files
    parameters_pro.create_file(parameters_file)
    function_pro.create_file(function_file)
    group_pro.create_file(group_file, tags_dict["air_gaps_tag"] != None)
    constraint_pro.create_file(constraint_file)

    simulate(onelab_folder_path, model_mesh_file_path, thermal_template_file)

    if show_results:
        gmsh.open(map_pos_file)
        gmsh.fltk.run()
